@@ -25,6 +25,7 @@ sys.setdefaultencoding("utf-8")
 
 #from curve import CurveThrift
 from curve import CurveThrift
+from curve.ttypes import TalkException
 
 try:
     import simplejson as json
@@ -410,6 +411,48 @@ class LineClient(object):
 
         return None
 
+    def getGroupFromId(self, id):
+        for group in self.groups:
+            if group.id == id:
+                return group
+
+        return None
+
+    def getContactOrGroupFromId(self, id):
+        for contact in self.contacts:
+            if contact.id == id:
+                return contact
+
+        for group in self.groups:
+            if group.id == id:
+                return group
+
+        return None
+
+    def longPoll(self, count=10, func=None):
+        """Check is there any operations from LINE server"""
+        OT = CurveThrift.OperationType
+
+        operations = self.client_in.fetchOperations(self.revision, count)
+
+        for operation in operations:
+            if operation.type == OT.END_OF_OPERATION:
+                pass
+            elif operation.type == OT.SEND_MESSAGE:
+                pass
+            elif operation.type == OT.RECEIVE_MESSAGE:
+                message = operation.message
+                group_or_contact = self.getContactOrGroupFromId(message.to)
+
+                if func != None:
+                    func(group_or_contact, message)
+
+                yield (group_or_contact, message)
+            else:
+                print operation
+
+            self.revision = max(operation.revision, self.revision)
+
     def raise_error(self, msg):
         """Fix a error format"""
         raise Exception("Error: %s" % msg)
@@ -508,33 +551,6 @@ class LineClient(object):
         if self.check_auth():
             return self.client.sendMessage(seq, message)
 
-    def longPoll(self):
-        """Check is there any operations from LINE server"""
-        OT = CurveClient.OperationType
-
-        try:
-            operations = self.client_in.fetchOperations(self.revision, 50)
-        except EOFError:
-            # long-poll timeout
-            return
-        except TalkException as e:
-            if e.code == 8:
-                msg = "user logged in to another machine."
-                raise_error(msg)
-            else:
-                return
-
-        for operation in operations:
-            if operation.type == OT.END_OF_OPERATION:
-                pass
-            elif operation.type == OT.SEND_MESSAGE:
-                print operation.message.text
-                pass
-            elif operation.type == OT.RECEIVE_MESSAGE:
-                pass
-
-            self.revision = max(operation.revision, self.revision)
-
     def _getLastOpRevision(self):
         if self.check_auth():
             return self.client.getLastOpRevision()
